import requests
from bs4 import BeautifulSoup
from bs4 import Tag
from android_app.database import insert_general_table

counter = 0

def get_unopened(raw):
    val = None
    if raw == '(Unopened/Opened)' or raw == '-':
        val = None
    elif "unopened" in raw.lower():
        val = False
    elif "opened" in raw.lower():
        val = True
    # print(str(val) + " for " + raw + " and " + str("opened" in raw.lower()) + " and " + str("unopened" in raw.lower()))
    return val

def get_lower_and_upper_range(raw):
    if '(' in raw:
        raw = raw[0: raw.index('(')]
    # Fix this case
    if '--' in raw:
        return None
    if '-' == raw:
        return None
    # Fix this case
    if "Year" in raw and "Months" in raw:
        return None

    if "date" in raw:
        raw = ' '.join(raw.split()[0:2])

    if raw == 'Same Day':
        return (1, 1, 'Day')
    elif '-' in raw:
        index = raw.index('-')
        return (int(raw[0:index]), int(raw[index + 1:index + 2]), raw[index + 3:].replace(" ", ""))
    else:
        index = raw.index(' ')
        return (int(raw[0:index]), None, raw[index + 1:].replace(" ", ""))


def scrape_category_of_items(URL, category, subcategory):
    r = requests.get(url=URL)

    soup = BeautifulSoup(r.text)

    for list in soup.find_all('div', {'class': 'arrow-list'}):
        for li in filter(lambda x: type(x) is Tag, list.ul.contents):
            name = li.a.contents[0]
            url = li.a['href']
            scrape_single_item_from_page(url, name, category, subcategory)

# Working for fruit pages
def scrape_single_item_from_page(URL, general_name, category,subcategory):
    global counter
    r = requests.get(url=URL)

    soup = BeautifulSoup(r.text, "html.parser")
    categories = []
    items = []

    possible_storages = ['Counter', 'Refrigerator', 'Freezer', 'Pantry', 'Shelf', 'Fridge']

    for tr in filter(lambda x: type(x) is Tag, soup.select('div table')[0].tbody.children):
        current_contents = list(filter(lambda x: type(x) is Tag, tr.contents))

        if len(current_contents) == 0:
            if len(tr.contents) != 0 and "opened" in str(tr.contents[0]).lower():
                categories[0] = str(tr.contents[0])
            continue
        if len(current_contents) > 1 and len(current_contents[1].contents) != 0 and str(current_contents[1].contents[0]) in possible_storages:
            categories.clear()
            for th in current_contents:
                categories.extend(list(map(lambda x: str(x), th.contents)))
            if len(current_contents[0].contents) == 0:
                categories.insert(0, '-')
        else:
            if "Date" in current_contents[1].contents[0]:
                continue
            name = current_contents[0].contents[0].contents[0]
            for i in range(1, len(categories)):
                unopened = get_unopened(categories[0])
                ranges = get_lower_and_upper_range(current_contents[i].contents[0])
                if ranges is None:
                    pass
                    print("Unable to add item " + name)
                else:
                    item = (name, counter, category, subcategory, categories[i], unopened, ranges[0], ranges[1], ranges[2])
                    items.append(items)
                    insert_general_table(item)
                    print("Inserted: " + str(item))
                    counter += 1


if __name__ == '__main__':
    scrape_category_of_items("https://www.eatbydate.com/fruits/fresh/", "Fruits", "Fresh Fruits")


import kivy
from kivy.core.window import Window


# Determines direction of user swipe
# set_initial must be called before get_swipe_direction
# 1 / threshold represents the fraction of your window's width that must dragged over to justify a swipe
class SwipeListener:
    def __init__(self, threshold=10):
        self._threshold = threshold
        self.__x1 = None

    def set_initial(self, x1):
        self.__x1 = x1

    # Returns 'left', 'right' if the absolute difference between x1 and x2 are >= threshold,
    # otherwise None is returned
    def get_swipe_direction(self, x2):
        if self.__x1 is None:
            return None

        if abs(x2 - self.__x1) > Window.size[0] / self._threshold:
            if x2 - self.__x1 > 0:
                return 'right'
            return 'left'
        return None


def valid_string(str):
    if not str:
        return False
    not_empty = False
    # print(str)
    for c in str:
        if (c <= 'z' and c >= 'a') or (c <= 'Z' and c >= 'A'):
            not_empty = True
    return not_empty


class Produce:

    def __init__(self, item):
        self.__itemName, self.__id, self.__category, self.__subcategory, self.__storageType, self.__unopened, \
        self.__expirationLowerBound, self.__expirationUpperBound, self.__expirationUnitType, self.__expirationDate = item

    def __getitem__(self, item):
        return getattr(self, 'expirationLowerBound')

    @property
    def itemName(self):
        return self.__itemName

    @property
    def expirationLowerBound(self):
        return self.__expirationLowerBound

    @property
    def expirationUnitType(self):
        return self.__expirationUnitType

    @property
    def expirationDate(self):
        return self.__expirationDate

    @property
    def id(self):
        return self.__id

    def return_as_tuple(self):
        return (self.__itemName, self.__id, self.__category, self.__subcategory, self.__storageType, self.__unopened, \
        self.__expirationLowerBound, self.__expirationUpperBound, self.__expirationUnitType, self.__expirationDate)


import sqlite3
from sqlite3 import Error
import numpy as np
from datetime import datetime as dt, date, timedelta
import time
import os.path

# ----------------- HELPER FUNCTIONS ----------------- #


# General function that executes sql code given the connection and sql
# Optional parameter for data used in the query
# Optional parameter for not needing to execute a db commit (when not modifying data)
# Returns cursor on success, None otherwise
def execute_sql(connection, sql, data=None, commit=True):
    try:
        curs = connection.cursor()
        if data is None:
            curs.execute(sql)
        else:
            curs.execute(sql, data)
        if commit:
            connection.commit()
        return curs
    except Error as err:
        print("--------- UNABLE TO SETUP CONNECTION -----")
        print(err)
        return None


# ----------------- CREATION FUNCTIONS ----------------- #


# Takes in the name of the db file and returns a connection to that db file.
def create_connection(db_file):
    print("CREATING CONNECTIONG $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$")
    connection = None

    try:
        print("WTFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF")
        print(os.path.join(os.path.dirname(__file__), db_file))
        connection = sqlite3.connect(os.path.join(os.path.dirname(__file__), db_file))
        return connection
        # print(sqlite3.version) # can use to print version
    except Error as err:
        print(err)

    return connection


# Creates the general items table in the expirations.db file
def create_general_table():
    sql_create_general_items_table = """ CREATE TABLE IF NOT EXISTS general_items (
                                        itemName varchar,
                                        id integer NOT NULL PRIMARY KEY,
                                        category integer NOT NULL,
                                        subcategory integer,
                                        storageType integer,
                                        unopened boolean,
                                        expirationLowerBound integer NOT NULL,
                                        expirationUpperBound integer,
                                        expirationUnitType varchar NOT NULL
                                    );"""

    connection = create_connection("expirations.db")

    if connection is not None:
        execute_sql(connection, sql_create_general_items_table, commit=False)
    else:
        print("Unable to create expirations.db connection.")


# Creates the user items table in the useritems.db file
def create_user_table():
    sql_create_user_items_table = """ CREATE TABLE IF NOT EXISTS user_items (
                                    itemName varchar NOT NULL,
                                    id integer PRIMARY KEY,
                                    category integer NOT NULL,
                                    subcategory integer,
                                    storageType integer,
                                    unopened boolean,
                                    expirationLowerBound integer NOT NULL,
                                    expirationUpperBound integer,
                                    expirationUnitType varchar NOT NULL,
                                    expirationDate datetime NOT NULL
                                    );"""

    connection = create_connection("useritems.db")

    if connection is not None:
        execute_sql(connection, sql_create_user_items_table, commit=False)
    else:
        print("Unable to create useritems.db connection.")


# Creates the categories table in the categories.db file
def create_category_table():
    sql_create_category_table = """ CREATE TABLE IF NOT EXISTS categories (
                                    id integer PRIMARY KEY,
                                    category varchar NOT NULL
                                    );"""

    connection = create_connection("categories.db")

    if connection is not None:
        execute_sql(connection, sql_create_category_table, commit=False)
    else:
        print("Unable to create categories.db connection.")


# Creates the subcategories table in the subcategories.db file
def create_subcategory_table():
    sql_create_subcategory_table = """ CREATE TABLE IF NOT EXISTS subcategories(
                                        id integer PRIMARY KEY,
                                        subcategory varchar NOT NULL
                                    );"""

    connection = create_connection("subcategories.db")

    if connection is not None:
        execute_sql(connection, sql_create_subcategory_table, commit=False)
    else:
        print("Unable to create subcategories.db connection.")


# Creates the storagetype table in the storagetypes.db file
def create_storage_type_table():
    sql_create_storage_type_table = """ CREATE TABLE IF NOT EXISTS storagetype(
                                        id integer PRIMARY KEY,
                                        storagetype varchar NOT NULL
                                    );"""

    connection = create_connection("storagetypes.db")

    if connection is not None:
        execute_sql(connection, sql_create_storage_type_table, commit=False)
    else:
        print("Unable to create storagetypes.db connection.")


# ----------------- INSERTION FUNCTIONS ----------------- #


# Inserts item in general_items table in expirations.db file
# Returns True on successful insertion, returns False otherwise
def insert_general_table(item):
    sql_insert_general_table = """ INSERT INTO general_items (itemName, id, category, subcategory, storageType, 
                                                            unopened, expirationLowerBound, expirationUpperBound,
                                                            expirationUnitType) VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?)"""

    connection = create_connection("expirations.db")

    if connection is not None:
        return True if execute_sql(connection, sql_insert_general_table, item) is not None else False
    else:
        print("Unable to create expirations.db connection.")
        return False


# Inserts item in user_items table in useritems.db file
# Returns True on successful insertion, returns False otherwise
def insert_user_table(item):

    normal_days = 0

    if item[8] == 'Days' or item[8] == 'days':
        normal_days = item[6]
    elif item[8] == 'Weeks' or item[8] == 'weeks':
        normal_days = item[6] * 7
    elif item[8] == 'Months' or item[8] == 'months':
        normal_days = item[6] * 30
    elif item[8] == 'Years' or item[8] == 'years':
        normal_days = item[6] * 365

    expirationDate = date.today() + timedelta(days=normal_days)
    item = list(item)
    item.append(expirationDate)

    results = query_all_user_item()
    i = 0
    for res in results:
        if res[1] == i:
            i += 1
        else:
            break

    item[1] = i
    item = tuple(item)

    sql_insert_user_table = """INSERT INTO user_items (itemName, id, category, subcategory, storageType, 
                                                            unopened, expirationLowerBound, expirationUpperBound,
                                                            expirationUnitType, expirationDate) VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"""

    connection = create_connection("useritems.db")

    if connection is not None:
        return item[1] if execute_sql(connection, sql_insert_user_table, item) is not None else False
    else:
        print("Unable to create useritems.db connection.")
        return False


# Inserts category in categories table in categories.db file
# Returns True on successful insertion, returns False otherwise
def insert_category_table(category):
    sql_insert_category_table = """INSERT INTO categories (id, category) VALUES(?, ?)"""

    connection = create_connection("categories.db")

    if connection is not None:
        return True if execute_sql(connection, sql_insert_category_table, category) is not None else False
    else:
        print("Unable to create categories.db connection.")
        return False


# Inserts subcategory in subcategories table in subcategories.db file
# Returns True on successful insertion, returns False otherwise
def insert_subcategory_table(subcategory):
    sql_insert_subcategory_table = """INSERT INTO subcategories (id, subcategory) VALUES(?, ?)"""

    connection = create_connection("subcategories.db")

    if connection is not None:
        return True if execute_sql(connection, sql_insert_subcategory_table, subcategory) is not None else False
    else:
        print("Unable to create subcategories.db connection.")
        return False


# Inserts storage_type in storagetype table in storagetypes.db
# Returns True on successful insertion, returns False otherwise
def insert_storage_type_table(storage_type):
    sql_insert_storage_type_table = """INSERT INTO storagetype (id, storagetype) VALUES(?, ?)"""

    connection = create_connection("storagetypes.db")

    if connection is not None:
        return True if execute_sql(connection, sql_insert_storage_type_table, storage_type) is not None else False
    else:
        print("Unable to create storagetypes.db connection.")
        return False


# ----------------- UPDATE FUNCTIONS ----------------- #


# Updates item in general_items table in expirations.db file
def update_general_table(item):
    sql_update_general_table = """ UPDATE general_items 
                                    SET id = ? ,
                                        category = ? ,
                                        subcategory = ? ,
                                        storageType = ? ,
                                        unopened = ? ,
                                        expirationLowerBound = ? ,
                                        expirationUpperBound = ? ,
                                        expirationUnitType = ?
                                    WHERE itemName = ?"""

    connection = create_connection("expirations.db")

    if connection is not None:
        return True if execute_sql(connection, sql_update_general_table, item) is not None else False
    else:
        print("Unable to create expirations.db connection.")
        return False


# Updates item in user_items table in useritems.db file
def update_user_table(item):
    sql_update_user_table = """ UPDATE user_items 
                                        SET id = ? ,
                                            category = ? ,
                                            subcategory = ? ,
                                            storageType = ? ,
                                            unopened = ? ,
                                            expirationLowerBound = ? ,
                                            expirationUpperBound = ? ,
                                            expirationUnitType = ?,
                                            expirationDate = ?
                                        WHERE itemName = ?"""

    connection = create_connection("useritems.db")

    if connection is not None:
        return True if execute_sql(connection, sql_update_user_table, item) is not None else False
    else:
        print("Unable to create useritems.db connection.")
        return False


# Updates category in categories table in categories.db file
def update_category_table(category):
    sql_update_category_table = """UPDATE categories 
                                    SET category = ?
                                    WHERE id = ?"""

    connection = create_connection("categories.db")

    if connection is not None:
        return True if execute_sql(connection, sql_update_category_table, category) is not None else False
    else:
        print("Unable to create categories.db connection.")
        return False


# Updates subcategory in subcategories table in subcategories.db file
def update_subcategory_table(subcategory):
    sql_update_subcategory_table = """UPDATE subcategories 
                                    SET subcategory = ?
                                    WHERE id = ?"""

    connection = create_connection("subcategories.db")

    if connection is not None:
        return True if execute_sql(connection, sql_update_subcategory_table, subcategory) is not None else False
    else:
        print("Unable to create subcategories.db connection.")
        return False


# Updates storage_type in storagetype table in storagetypes.db
def update_storage_type_table(storage_type):
    sql_update_storage_type_table = """UPDATE storagetype
                                    SET storagetype = ?
                                    WHERE id = ?"""

    connection = create_connection("storagetypes.db")

    if connection is not None:
        return True if execute_sql(connection, sql_update_storage_type_table, storage_type) is not None else False
    else:
        print("Unable to create storagetypes.db connection.")
        return False


# ----------------- QUERY FUNCTIONS ----------------- #


def query_all_user_item():
    sql_query_user_item = """SELECT * FROM user_items"""

    connection = create_connection("useritems.db")
    print("%%%%%%%%%%%%")

    if connection is not None:
        curs = execute_sql(connection, sql_query_user_item, commit=False)
        results = curs.fetchall()
        return results
    else:
        print("Unable to create useritems.db connection.")
        return None


# Queries for a user_item based on if item id matches input
# Returns table results as 2d array from the query
def query_user_item_by_id(id):
    sql_query_user_item = """SELECT * FROM user_items WHERE id = ?"""

    connection = create_connection("useritems.db")

    if connection is not None:
        curs = execute_sql(connection, sql_query_user_item, (id,), commit=False)
        results = curs.fetchall()
        return results
    else:
        print("Unable to create useritems.db connection.")
        return None


# Queries for a user_item based on if item name contains input
# Returns
def query_user_item_by_name(name):
    sql_query_user_item = """SELECT * FROM user_items WHERE itemName LIKE '%'||?||'%'"""

    connection = create_connection("useritems.db")

    if connection is not None:
        curs = execute_sql(connection, sql_query_user_item, (name,), commit=False)
        results = curs.fetchall()
        return results
    else:
        print("Unable to create useritems.db connection.")
        return None


# ----------------- DELETE FUNCTIONS ----------------- #


# Deletes a user_item according to its id from useritems.db
def delete_user_item(id):
    sql_delete_user_item = """DELETE FROM user_items WHERE id = ?"""

    connection = create_connection("useritems.db")

    if connection is not None:
        return True if execute_sql(connection, sql_delete_user_item, (id,)) is not None else False
    else:
        print("Unable to create useritems.db connection.")
        return False


# Deletes all user_items in the table from useritems.db
def delete_all_user_items():
    sql_delete_all_items = """DELETE FROM user_items"""

    connection = create_connection("useritems.db")

    if connection is not None:
        return True if execute_sql(connection, sql_delete_all_items) is not None else False
    else:
        print("Unable to create useritems.db connection.")
        return False


# Deletes all general_items in the table from expirations.db
def delete_all_general_items():
    sql_delete_all_items = """DELETE FROM general_items"""

    connection = create_connection("expirations.db")

    if connection is not None:
        return True if execute_sql(connection, sql_delete_all_items) is not None else False
    else:
        print("Unable to create expirations.db connection.")
        return False


# Deletes all categories in the table from categories.db
def delete_all_categories():
    sql_delete_all_items = """DELETE FROM categories"""

    connection = create_connection("categories.db")

    if connection is not None:
        return True if execute_sql(connection, sql_delete_all_items) is not None else False
    else:
        print("Unable to create categories.db connection.")
        return False


# Deletes all subcategories in the table from subcategories.db
def delete_all_subcategories():
    sql_delete_all_items = """DELETE FROM general_items"""

    connection = create_connection("subcategories.db")

    if connection is not None:
        return True if execute_sql(connection, sql_delete_all_items) is not None else False
    else:
        print("Unable to create subcategories.db connection.")
        return False


# Deletes all storage types in the table from storagetypes.db
def delete_all_storage_types():
    sql_delete_all_items = """DELETE FROM storagetypes"""

    connection = create_connection("storagetypes.db")

    if connection is not None:
        return True if execute_sql(connection, sql_delete_all_items) is not None else False
    else:
        print("Unable to create storagetypes.db connection.")
        return False


def levenshtein(s, t):
    rows = len(s)+1
    cols = len(t)+1
    distance = np.zeros((rows,cols), dtype = int)

    for i in range(1, rows):
        for k in range(1,cols):
            distance[i][0] = i
            distance[0][k] = k

    for col in range(1, cols):
        for row in range(1, rows):
            if s[row-1] == t[col-1]:
                cost = 0
            else:
                cost = 2
            distance[row][col] = min(distance[row-1][col] + 1,      # Cost of deletions
                                 distance[row][col-1] + 1,          # Cost of insertions
                                 distance[row-1][col-1] + cost)     # Cost of substitutions
    Ratio = ((len(s)+len(t)) - distance[row][col]) / (len(s)+len(t))
    return Ratio


def match_item(raw_item):
    sql_query_all_item = """SELECT * FROM general_items"""

    connection = create_connection("expirations.db")

    if connection is not None:
        curs = execute_sql(connection, sql_query_all_item, (), commit=False)
        results = curs.fetchall()
        max = -1
        curr = None
        for i in results:
            ratio = levenshtein(i[0], raw_item).item()
            if ratio > max:
                curr = i
                max = ratio
        return curr
    else:
        print("Unable to create expirations.db.")
        return None


if __name__ == "__main__":
    # create_general_table()
    create_user_table()



import os

os.system("sudo apt-get install xclip xsel")
os.system("sudo apt install tesseract-ocr")
os.system("sudo apt-get remove gstreamer1.0-alsa gstreamer1.0-libav gstreamer1.0-plugins-bad gstreamer1.0-plugins-base gstreamer1.0-plugins-good gstreamer1.0-plugins-ugly gstreamer1.0-pulseaudio libgstreamer-plugins-bad1.0-0 libgstreamer-plugins-base1.0-0 libgstreamer-plugins-good1.0-0 libgstreamer1.0-0")



import unittest
import db

blueberries = ('blueberries', 21, 'fruit', 'berries', 'fridge', False, 2, 4, 'days')
wild_blueberries = ('wild blueberries', 24, 'fruit', 'berries', 'fridge', False, 1, 5, 'days')
strawberries = ('strawberries', 22, 'fruit', 'berries', 'fridge', False, 1, 3, 'weeks')
blackberries = ('blackberries', 23, 'fruit', 'berries', 'fridge', False, 2, 5, 'months')
blue_lollipops = ('blue lollipops', 25, 'candy', None, None, False, 999, None, 'years')


class TestUserItems(unittest.TestCase):
    def setUp(self):
        db.create_user_table()

    def test_add_unique_item_and_remove(self):
        self.assertTrue(db.insert_user_table(blueberries))
        self.assertTrue(db.delete_user_item(blueberries[1]))

    def test_delete_all(self):
        self.assertTrue(db.insert_user_table(blueberries))
        self.assertTrue(db.insert_user_table(strawberries))
        self.assertTrue(db.insert_user_table(blackberries))
        self.assertTrue(db.delete_all_user_items())

    def test_query_by_id(self):
        self.assertTrue(db.insert_user_table(strawberries))
        self.assertTrue(db.insert_user_table(blueberries))
        self.assertTrue(db.insert_user_table(blackberries))
        result = db.query_user_item_by_id(blueberries[1])
        self.assertIsNotNone(result)
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0][1], blueberries[1])

    def test_query_multiple_items_by_name(self):
        self.assertTrue(db.insert_user_table(blueberries))
        self.assertTrue(db.insert_user_table(wild_blueberries))
        self.assertTrue(db.insert_user_table(blue_lollipops))
        result = db.query_user_item_by_name('blue')
        self.assertIsNotNone(result)
        self.assertEqual(len(result), 3)

    def tearDown(self):
        db.delete_all_user_items()


if __name__ == "__main__":
    unittest.main()



import re

import kivy
from kivy.app import App
from kivy.clock import Clock
from kivy.lang import Builder
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.screenmanager import ScreenManager, Screen
from kivy.uix.scrollview import ScrollView
from kivy import utils
from producetracker.utilities import SwipeListener, Produce, valid_string
from productetracker.install import installer
from producetracker.database import *
import os.path
import sys

import kivy.resources
kivy.resources.resource_add_path(os.path.join(os.path.dirname(__file__), 'resources'))

from PIL import Image

import pytesseract

pytesseract.tesseract_cmd = r'../pytesseract/tesseract'

kivy.require('1.11.1')


class Manager(ScreenManager):

    swipe_listener = SwipeListener(5)

    def on_touch_down(self, touch):
        if self.current != 'input':
            self.swipe_listener.set_initial(touch.x)
        super(Manager, self).on_touch_down(touch)  # Completes other on_touch_down arguments (buttons)

    def on_touch_up(self, touch, *args):
        swipe_direction = None
        if self.current != 'input':
            swipe_direction = self.swipe_listener.get_swipe_direction(touch.x)
        if swipe_direction == 'left':
            self.current = self.next()
            if self.current == 'input':
                self.current = self.next()
        elif swipe_direction == 'right':
            self.current = self.previous()
            if self.current == 'input':
                self.current = self.previous()


class LandingPage(Screen):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def capture(self):
        camera = self.ids['camera']
        camera.export_to_png("IMG_SCANNED.png")


class PantryPage(Screen):
    produce_list = []

    # queries all produce from the database and appends them to produce_list
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        all_items = query_all_user_item()

        for item in all_items:
            self.produce_list.append(Produce(item))

    def on_enter(self, *args):
        self.ids.nav_bar.ids.pantry_button.canvas.children[0].children[0].rgba = utils.get_color_from_hex('#385E3C')
        self.reset_list()

    # sorts produce_list, clears the scroll_menu, then adds all items from produce_list to scroll_menu+
    def reset_list(self):
        self.produce_list = sorted(self.produce_list,
                                   key=lambda x: int((dt.fromisoformat(x.expirationDate) - dt.today()).days),
                                   reverse=False)
        self.ids.scroll_menu.ids.grid_layout.clear_widgets()
        for item in self.produce_list:
            self.ids.scroll_menu.add_to_menu(str(item.itemName), (
                    str((dt.fromisoformat(item.expirationDate) - dt.today()).days + 1) + ' day(s)'), item.id)

    def reset_title(self, *args):
        self.ids.title_text.text = 'Pantry'
        self.ids.title_text.color = utils.get_color_from_hex('#000000')

    def read_image(self):
        ret_str = pytesseract.image_to_string(Image.open('IMG_SCANNED.png'))
        ret_str = ret_str.lower()
        ret_str = ret_str.lower()

        re.sub(r'[^A-Za-z ]+', '', ret_str)
        list_entries = ret_str.splitlines()
        list_entries = list(filter(lambda item: valid_string(item), list_entries))
        if len(list_entries) == 0:
            self.ids.title_text.text = 'Scan Failed!'
            self.ids.title_text.color = utils.get_color_from_hex('#FFFFFF')
            Clock.schedule_once(self.parent.children[0].reset_title, 3)
        for item in list_entries:
            self.text_passed(text=item)

    def text_passed(self, text=None):
        if text is None:
            ret_item = match_item(self.ids.produce_input.text)
        else:
            ret_item = match_item(text)
        if ret_item is not None:
            id_ret = insert_user_table(ret_item)
            self.produce_list.append(Produce(query_user_item_by_id(id_ret)[0]))
            self.reset_list()
            self.ids.title_text.text = 'Produce Added Successfully!'
            self.ids.title_text.color = utils.get_color_from_hex('#FFFFFF')
            Clock.schedule_once(self.parent.children[0].reset_title, 3)
        else:
            self.ids.title_text.text = 'Failed to Add Produce!'
            self.ids.title_text.color = utils.get_color_from_hex('#FFFFFF')
            Clock.schedule_once(self.parent.children[0].reset_title, 3)


class IdeasPage(Screen):
    def on_enter(self, *args):
        self.ids.nav_bar.ids.ideas_button.canvas.children[0].children[0].rgba = utils.get_color_from_hex('#385E3C')


class SettingsPage(Screen):
    def on_enter(self, *args):
        self.ids.nav_bar.ids.settings_button.canvas.children[0].children[0].rgba = utils.get_color_from_hex('#385E3C')


class AboutPage(Screen):
    def on_enter(self, *args):
        self.ids.nav_bar.ids.about_button.canvas.children[0].children[0].rgba = utils.get_color_from_hex('#385E3C')


class InputPage(Screen):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def read_image(self):
        ret_str = pytesseract.image_to_string(Image.open('IMG_SCANNED.png'))
        ret_str.split()
        re.sub(r'[^A-Za-z ]+', '', ret_str)
        list_entries = ret_str.splitlines()
        list_entries = list(filter(lambda item: valid_string(item), list_entries))
        for item in list_entries:
            self.text_entered(text=item)

    def text_entered(self, text=None):
        if text is None:
            ret_item = match_item(self.ids.produce_input.text)
        else:
            ret_item = match_item(text)
        if ret_item is not None:
            id_ret = insert_user_table(ret_item)
            self.parent.children[0].produce_list.append(Produce(query_user_item_by_id(id_ret)[0]))
            self.parent.children[0].reset_list()
            self.parent.children[0].ids.title_text.text = 'Produce Added Successfully!'
            self.parent.children[0].ids.title_text.color = utils.get_color_from_hex('#FFFFFF')
            Clock.schedule_once(self.parent.children[0].reset_title, 3)
        else:
            self.parent.children[0].ids.title_text.text = 'Failed to Add Produce!'
            self.parent.children[0].ids.title_text.color = utils.get_color_from_hex('#FFFFFF')
            Clock.schedule_once(self.parent.children[0].reset_title, 3)


class MenuItem(BoxLayout):
    def __init__(self, name, time_remaining, quantity=1, **kwargs):
        super().__init__(**kwargs)
        self.ids.produce_label.text = name
        self.ids.expiration_label.text = time_remaining

    def remove(self, *args):
        calc_index = len(self.parent.children) - 1 - self.parent.children.index(self)
        delete_user_item(self.parent.parent.parent.parent.produce_list[calc_index].id)
        self.parent.parent.parent.parent.produce_list.pop(calc_index)  # removes item at calc_index from produce_list
        self.parent.remove_widget(self)


class ScrollMenu(ScrollView):

    # Adds a MenuItem to the ScrollMenu
    def add_to_menu(self, name, time_remaining, quantity=1):
        new_menu_item = MenuItem(name, time_remaining, quantity)
        self.ids.grid_layout.add_widget(new_menu_item)

        # Change text color to red if expiration below threshold
        if int(time_remaining.split()[0]) <= 3:
            self.ids.grid_layout.children[0].ids.produce_label.color = utils.get_color_from_hex("#C40233")
            self.ids.grid_layout.children[0].ids.expiration_label.color = utils.get_color_from_hex("#C40233")


class BadApplesApp(App):
    def build(self):
        root = Builder.load_file(os.path.join(os.path.dirname(__file__), 'style.kv'))
        return root

def main():
    installer()
    BadApplesApp().run()

if __name__ == "__main__":
    main()
